@@ -10,12 +10,14 @@
 #include "android/keycodes.h"
 #include "common.h"
 
+#define CONTROL_MSG_MAX_SIZE (1 << 18) // 256k
+
 #define CONTROL_MSG_INJECT_TEXT_MAX_LENGTH 300
-#define CONTROL_MSG_CLIPBOARD_TEXT_MAX_LENGTH 4092
-#define CONTROL_MSG_SERIALIZED_MAX_SIZE \
-    (4 + CONTROL_MSG_CLIPBOARD_TEXT_MAX_LENGTH)
+// type: 1 byte; paste flag: 1 byte; length: 4 bytes
+#define CONTROL_MSG_CLIPBOARD_TEXT_MAX_LENGTH (CONTROL_MSG_MAX_SIZE - 6)
 
 #define POINTER_ID_MOUSE UINT64_C(-1);
+#define POINTER_ID_VIRTUAL_FINGER UINT64_C(-2);
 
 enum control_msg_type {
     CONTROL_MSG_TYPE_INJECT_KEYCODE,
@@ -43,6 +45,7 @@ struct control_msg {
         struct {
             enum android_keyevent_action action;
             enum android_keycode keycode;
+            uint32_t repeat;
             enum android_metastate metastate;
         } inject_keycode;
         struct {
@@ -70,7 +73,7 @@ struct control_msg {
     };
 };
 
-// buf size must be at least CONTROL_MSG_SERIALIZED_MAX_SIZE
+// buf size must be at least CONTROL_MSG_MAX_SIZE
 // return the number of bytes written
 size_t
 control_msg_serialize(const struct control_msg *msg, unsigned char *buf);
