@@ -1,6 +1,7 @@
 #include "input_manager.h"
 
 #include <assert.h>
+#include <SDL2/SDL_keycode.h>
 
 #include "config.h"
 #include "event_converter.h"
@@ -10,6 +11,71 @@
 static const int ACTION_DOWN = 1;
 static const int ACTION_UP = 1 << 1;
 
+#define SC_SDL_SHORTCUT_MODS_MASK (KMOD_CTRL | KMOD_ALT | KMOD_GUI)
+
+static inline uint16_t
+to_sdl_mod(unsigned mod) {
+    uint16_t sdl_mod = 0;
+    if (mod & SC_MOD_LCTRL) {
+        sdl_mod |= KMOD_LCTRL;
+    }
+    if (mod & SC_MOD_RCTRL) {
+        sdl_mod |= KMOD_RCTRL;
+    }
+    if (mod & SC_MOD_LALT) {
+        sdl_mod |= KMOD_LALT;
+    }
+    if (mod & SC_MOD_RALT) {
+        sdl_mod |= KMOD_RALT;
+    }
+    if (mod & SC_MOD_LSUPER) {
+        sdl_mod |= KMOD_LGUI;
+    }
+    if (mod & SC_MOD_RSUPER) {
+        sdl_mod |= KMOD_RGUI;
+    }
+    return sdl_mod;
+}
+
+static bool
+is_shortcut_mod(struct input_manager *im, uint16_t sdl_mod) {
+    // keep only the relevant modifier keys
+    sdl_mod &= SC_SDL_SHORTCUT_MODS_MASK;
+
+    assert(im->sdl_shortcut_mods.count);
+    assert(im->sdl_shortcut_mods.count < SC_MAX_SHORTCUT_MODS);
+    for (unsigned i = 0; i < im->sdl_shortcut_mods.count; ++i) {
+        if (im->sdl_shortcut_mods.data[i] == sdl_mod) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+void
+input_manager_init(struct input_manager *im,
+                   const struct scrcpy_options *options)
+{
+    im->control = options->control;
+    im->forward_key_repeat = options->forward_key_repeat;
+    im->prefer_text = options->prefer_text;
+    im->forward_all_clicks = options->forward_all_clicks;
+    im->legacy_paste = options->legacy_paste;
+
+    const struct sc_shortcut_mods *shortcut_mods = &options->shortcut_mods;
+    assert(shortcut_mods->count);
+    assert(shortcut_mods->count < SC_MAX_SHORTCUT_MODS);
+    for (unsigned i = 0; i < shortcut_mods->count; ++i) {
+        uint16_t sdl_mod = to_sdl_mod(shortcut_mods->data[i]);
+        assert(sdl_mod);
+        im->sdl_shortcut_mods.data[i] = sdl_mod;
+    }
+    im->sdl_shortcut_mods.count = shortcut_mods->count;
+
+    im->vfinger_down = false;
+}
+
 static void
 send_keycode(struct controller *controller, enum android_keycode keycode,
              int actions, const char *name) {
@@ -18,6 +84,7 @@ send_keycode(struct controller *controller, enum android_keycode keycode,
     msg.type = CONTROL_MSG_TYPE_INJECT_KEYCODE;
     msg.inject_keycode.keycode = keycode;
     msg.inject_keycode.metastate = 0;
+    msg.inject_keycode.repeat = 0;
 
     if (actions & ACTION_DOWN) {
         msg.inject_keycode.action = AKEY_EVENT_ACTION_DOWN;
@@ -70,6 +137,16 @@ action_menu(struct controller *controller, int actions) {
     send_keycode(controller, AKEYCODE_MENU, actions, "MENU");
 }
 
+static inline void
+action_copy(struct controller *controller, int actions) {
+    send_keycode(controller, AKEYCODE_COPY, actions, "COPY");
+}
+
+static inline void
+action_cut(struct controller *controller, int actions) {
+    send_keycode(controller, AKEYCODE_CUT, actions, "CUT");
+}
+
 // turn the screen on if it was off, press BACK otherwise
 static void
 press_back_or_turn_screen_on(struct controller *controller) {
@@ -101,16 +178,6 @@ collapse_notification_panel(struct controller *controller) {
     }
 }
 
-static void
-request_device_clipboard(struct controller *controller) {
-    struct control_msg msg;
-    msg.type = CONTROL_MSG_TYPE_GET_CLIPBOARD;
-
-    if (!controller_push_msg(controller, &msg)) {
-        LOGW("Could not request device clipboard");
-    }
-}
-
 static void
 set_device_clipboard(struct controller *controller, bool paste) {
     char *text = SDL_GetClipboardText();
@@ -210,6 +277,10 @@ rotate_client_right(struct screen *screen) {
 void
 input_manager_process_text_input(struct input_manager *im,
                                  const SDL_TextInputEvent *event) {
+    if (is_shortcut_mod(im, SDL_GetModState())) {
+        // A shortcut must never generate text events
+        return;
+    }
     if (!im->prefer_text) {
         char c = event->text[0];
         if (isalpha(c) || c == ' ') {
@@ -232,9 +303,39 @@ input_manager_process_text_input(struct input_manager *im,
     }
 }
 
+static bool
+simulate_virtual_finger(struct input_manager *im,
+                        enum android_motionevent_action action,
+                        struct point point) {
+    bool up = action == AMOTION_EVENT_ACTION_UP;
+
+    struct control_msg msg;
+    msg.type = CONTROL_MSG_TYPE_INJECT_TOUCH_EVENT;
+    msg.inject_touch_event.action = action;
+    msg.inject_touch_event.position.screen_size = im->screen->frame_size;
+    msg.inject_touch_event.position.point = point;
+    msg.inject_touch_event.pointer_id = POINTER_ID_VIRTUAL_FINGER;
+    msg.inject_touch_event.pressure = up ? 0.0f : 1.0f;
+    msg.inject_touch_event.buttons = 0;
+
+    if (!controller_push_msg(im->controller, &msg)) {
+        LOGW("Could not request 'inject virtual finger event'");
+        return false;
+    }
+
+    return true;
+}
+
+static struct point
+inverse_point(struct point point, struct size size) {
+    point.x = size.width - point.x;
+    point.y = size.height - point.y;
+    return point;
+}
+
 static bool
 convert_input_key(const SDL_KeyboardEvent *from, struct control_msg *to,
-                  bool prefer_text) {
+                  bool prefer_text, uint32_t repeat) {
     to->type = CONTROL_MSG_TYPE_INJECT_KEYCODE;
 
     if (!convert_keycode_action(from->type, &to->inject_keycode.action)) {
@@ -247,6 +348,7 @@ convert_input_key(const SDL_KeyboardEvent *from, struct control_msg *to,
         return false;
     }
 
+    to->inject_keycode.repeat = repeat;
     to->inject_keycode.metastate = convert_meta_state(mod);
 
     return true;
@@ -254,74 +356,52 @@ convert_input_key(const SDL_KeyboardEvent *from, struct control_msg *to,
 
 void
 input_manager_process_key(struct input_manager *im,
-                          const SDL_KeyboardEvent *event,
-                          bool control) {
+                          const SDL_KeyboardEvent *event) {
     // control: indicates the state of the command-line option --no-control
-    // ctrl: the Ctrl key
+    bool control = im->control;
 
-    bool ctrl = event->keysym.mod & (KMOD_LCTRL | KMOD_RCTRL);
-    bool alt = event->keysym.mod & (KMOD_LALT | KMOD_RALT);
-    bool meta = event->keysym.mod & (KMOD_LGUI | KMOD_RGUI);
-
-    // use Cmd on macOS, Ctrl on other platforms
-#ifdef __APPLE__
-    bool cmd = !ctrl && meta;
-#else
-    if (meta) {
-        // no shortcuts involve Meta on platforms other than macOS, and it must
-        // not be forwarded to the device
-        return;
-    }
-    bool cmd = ctrl; // && !meta, already guaranteed
-#endif
-
-    if (alt) {
-        // no shortcuts involve Alt, and it must not be forwarded to the device
-        return;
-    }
+    bool smod = is_shortcut_mod(im, event->keysym.mod);
 
     struct controller *controller = im->controller;
 
-    // capture all Ctrl events
-    if (ctrl || cmd) {
-        SDL_Keycode keycode = event->keysym.sym;
-        bool down = event->type == SDL_KEYDOWN;
+    SDL_Keycode keycode = event->keysym.sym;
+    bool down = event->type == SDL_KEYDOWN;
+    bool ctrl = event->keysym.mod & KMOD_CTRL;
+    bool shift = event->keysym.mod & KMOD_SHIFT;
+    bool repeat = event->repeat;
+
+    // The shortcut modifier is pressed
+    if (smod) {
         int action = down ? ACTION_DOWN : ACTION_UP;
-        bool repeat = event->repeat;
-        bool shift = event->keysym.mod & (KMOD_LSHIFT | KMOD_RSHIFT);
         switch (keycode) {
             case SDLK_h:
-                // Ctrl+h on all platform, since Cmd+h is already captured by
-                // the system on macOS to hide the window
-                if (control && ctrl && !meta && !shift && !repeat) {
+                if (control && !shift && !repeat) {
                     action_home(controller, action);
                 }
                 return;
             case SDLK_b: // fall-through
             case SDLK_BACKSPACE:
-                if (control && cmd && !shift && !repeat) {
+                if (control && !shift && !repeat) {
                     action_back(controller, action);
                 }
                 return;
             case SDLK_s:
-                if (control && cmd && !shift && !repeat) {
+                if (control && !shift && !repeat) {
                     action_app_switch(controller, action);
                 }
                 return;
             case SDLK_m:
-                // Ctrl+m on all platform, since Cmd+m is already captured by
-                // the system on macOS to minimize the window
-                if (control && ctrl && !meta && !shift && !repeat) {
+                if (control && !shift && !repeat) {
                     action_menu(controller, action);
                 }
                 return;
             case SDLK_p:
-                if (control && cmd && !shift && !repeat) {
+                if (control && !shift && !repeat) {
                     action_power(controller, action);
                 }
                 return;
             case SDLK_o:
-                if (control && cmd && down) {
+                if (control && !repeat && down) {
                     enum screen_power_mode mode = shift
                                                 ? SCREEN_POWER_MODE_NORMAL
                                                 : SCREEN_POWER_MODE_OFF;
@@ -329,67 +409,72 @@ input_manager_process_key(struct input_manager *im,
                 }
                 return;
             case SDLK_DOWN:
-                if (control && cmd && !shift) {
+                if (control && !shift) {
                     // forward repeated events
                     action_volume_down(controller, action);
                 }
                 return;
             case SDLK_UP:
-                if (control && cmd && !shift) {
+                if (control && !shift) {
                     // forward repeated events
                     action_volume_up(controller, action);
                 }
                 return;
             case SDLK_LEFT:
-                if (cmd && !shift && down) {
+                if (!shift && !repeat && down) {
                     rotate_client_left(im->screen);
                 }
                 return;
             case SDLK_RIGHT:
-                if (cmd && !shift && down) {
+                if (!shift && !repeat && down) {
                     rotate_client_right(im->screen);
                 }
                 return;
             case SDLK_c:
-                if (control && cmd && !shift && !repeat && down) {
-                    request_device_clipboard(controller);
+                if (control && !shift && !repeat) {
+                    action_copy(controller, action);
+                }
+                return;
+            case SDLK_x:
+                if (control && !shift && !repeat) {
+                    action_cut(controller, action);
                 }
                 return;
             case SDLK_v:
-                if (control && cmd && !repeat && down) {
-                    if (shift) {
-                        // store the text in the device clipboard and paste
-                        set_device_clipboard(controller, true);
-                    } else {
+                if (control && !repeat && down) {
+                    if (shift || im->legacy_paste) {
                         // inject the text as input events
                         clipboard_paste(controller);
+                    } else {
+                        // store the text in the device clipboard and paste
+                        set_device_clipboard(controller, true);
                     }
                 }
                 return;
             case SDLK_f:
-                if (!shift && cmd && !repeat && down) {
+                if (!shift && !repeat && down) {
                     screen_switch_fullscreen(im->screen);
                 }
                 return;
-            case SDLK_x:
-                if (!shift && cmd && !repeat && down) {
+            case SDLK_w:
+                if (!shift && !repeat && down) {
                     screen_resize_to_fit(im->screen);
                 }
                 return;
             case SDLK_g:
-                if (!shift && cmd && !repeat && down) {
+                if (!shift && !repeat && down) {
                     screen_resize_to_pixel_perfect(im->screen);
                 }
                 return;
             case SDLK_i:
-                if (!shift && cmd && !repeat && down) {
+                if (!shift && !repeat && down) {
                     struct fps_counter *fps_counter =
                         im->video_buffer->fps_counter;
                     switch_fps_counter_state(fps_counter);
                 }
                 return;
             case SDLK_n:
-                if (control && cmd && !repeat && down) {
+                if (control && !repeat && down) {
                     if (shift) {
                         collapse_notification_panel(controller);
                     } else {
@@ -398,7 +483,7 @@ input_manager_process_key(struct input_manager *im,
                 }
                 return;
             case SDLK_r:
-                if (control && cmd && !shift && !repeat && down) {
+                if (control && !shift && !repeat && down) {
                     rotate_device(controller);
                 }
                 return;
@@ -411,8 +496,28 @@ input_manager_process_key(struct input_manager *im,
         return;
     }
 
+    if (event->repeat) {
+        if (!im->forward_key_repeat) {
+            return;
+        }
+        ++im->repeat;
+    } else {
+        im->repeat = 0;
+    }
+
+    if (ctrl && !shift && keycode == SDLK_v && down && !repeat) {
+        if (im->legacy_paste) {
+            // inject the text as input events
+            clipboard_paste(controller);
+            return;
+        }
+        // Synchronize the computer clipboard to the device clipboard before
+        // sending Ctrl+v, to allow seamless copy-paste.
+        set_device_clipboard(controller, false);
+    }
+
     struct control_msg msg;
-    if (convert_input_key(event, &msg, im->prefer_text)) {
+    if (convert_input_key(event, &msg, im->prefer_text, im->repeat)) {
         if (!controller_push_msg(controller, &msg)) {
             LOGW("Could not request 'inject keycode'");
         }
@@ -427,7 +532,7 @@ convert_mouse_motion(const SDL_MouseMotionEvent *from, struct screen *screen,
     to->inject_touch_event.pointer_id = POINTER_ID_MOUSE;
     to->inject_touch_event.position.screen_size = screen->frame_size;
     to->inject_touch_event.position.point =
-        screen_convert_to_frame_coords(screen, from->x, from->y);
+        screen_convert_window_to_frame_coords(screen, from->x, from->y);
     to->inject_touch_event.pressure = 1.f;
     to->inject_touch_event.buttons = convert_mouse_buttons(from->state);
 
@@ -446,10 +551,18 @@ input_manager_process_mouse_motion(struct input_manager *im,
         return;
     }
     struct control_msg msg;
-    if (convert_mouse_motion(event, im->screen, &msg)) {
-        if (!controller_push_msg(im->controller, &msg)) {
-            LOGW("Could not request 'inject mouse motion event'");
-        }
+    if (!convert_mouse_motion(event, im->screen, &msg)) {
+        return;
+    }
+
+    if (!controller_push_msg(im->controller, &msg)) {
+        LOGW("Could not request 'inject mouse motion event'");
+    }
+
+    if (im->vfinger_down) {
+        struct point mouse = msg.inject_touch_event.position.point;
+        struct point vfinger = inverse_point(mouse, im->screen->frame_size);
+        simulate_virtual_finger(im, AMOTION_EVENT_ACTION_MOVE, vfinger);
     }
 }
 
@@ -465,15 +578,15 @@ convert_touch(const SDL_TouchFingerEvent *from, struct screen *screen,
     to->inject_touch_event.pointer_id = from->fingerId;
     to->inject_touch_event.position.screen_size = screen->frame_size;
 
-    int ww;
-    int wh;
-    SDL_GL_GetDrawableSize(screen->window, &ww, &wh);
+    int dw;
+    int dh;
+    SDL_GL_GetDrawableSize(screen->window, &dw, &dh);
 
     // SDL touch event coordinates are normalized in the range [0; 1]
-    int32_t x = from->x * ww;
-    int32_t y = from->y * wh;
+    int32_t x = from->x * dw;
+    int32_t y = from->y * dh;
     to->inject_touch_event.position.point =
-        screen_convert_to_frame_coords(screen, x, y);
+        screen_convert_drawable_to_frame_coords(screen, x, y);
 
     to->inject_touch_event.pressure = from->pressure;
     to->inject_touch_event.buttons = 0;
@@ -503,8 +616,9 @@ convert_mouse_button(const SDL_MouseButtonEvent *from, struct screen *screen,
     to->inject_touch_event.pointer_id = POINTER_ID_MOUSE;
     to->inject_touch_event.position.screen_size = screen->frame_size;
     to->inject_touch_event.position.point =
-        screen_convert_to_frame_coords(screen, from->x, from->y);
-    to->inject_touch_event.pressure = 1.f;
+        screen_convert_window_to_frame_coords(screen, from->x, from->y);
+    to->inject_touch_event.pressure =
+        from->type == SDL_MOUSEBUTTONDOWN ? 1.f : 0.f;
     to->inject_touch_event.buttons =
         convert_mouse_buttons(SDL_BUTTON(from->button));
 
@@ -513,13 +627,16 @@ convert_mouse_button(const SDL_MouseButtonEvent *from, struct screen *screen,
 
 void
 input_manager_process_mouse_button(struct input_manager *im,
-                                   const SDL_MouseButtonEvent *event,
-                                   bool control) {
+                                   const SDL_MouseButtonEvent *event) {
+    bool control = im->control;
+
     if (event->which == SDL_TOUCH_MOUSEID) {
         // simulated from touch events, so it's a duplicate
         return;
     }
-    if (event->type == SDL_MOUSEBUTTONDOWN) {
+
+    bool down = event->type == SDL_MOUSEBUTTONDOWN;
+    if (!im->forward_all_clicks && down) {
         if (control && event->button == SDL_BUTTON_RIGHT) {
             press_back_or_turn_screen_on(im->controller);
             return;
@@ -550,10 +667,36 @@ input_manager_process_mouse_button(struct input_manager *im,
     }
 
     struct control_msg msg;
-    if (convert_mouse_button(event, im->screen, &msg)) {
-        if (!controller_push_msg(im->controller, &msg)) {
-            LOGW("Could not request 'inject mouse button event'");
+    if (!convert_mouse_button(event, im->screen, &msg)) {
+        return;
+    }
+
+    if (!controller_push_msg(im->controller, &msg)) {
+        LOGW("Could not request 'inject mouse button event'");
+        return;
+    }
+
+    // Pinch-to-zoom simulation.
+    //
+    // If Ctrl is hold when the left-click button is pressed, then
+    // pinch-to-zoom mode is enabled: on every mouse event until the left-click
+    // button is released, an additional "virtual finger" event is generated,
+    // having a position inverted through the center of the screen.
+    //
+    // In other words, the center of the rotation/scaling is the center of the
+    // screen.
+#define CTRL_PRESSED (SDL_GetModState() & (KMOD_LCTRL | KMOD_RCTRL))
+    if ((down && !im->vfinger_down && CTRL_PRESSED)
+            || (!down && im->vfinger_down)) {
+        struct point mouse = msg.inject_touch_event.position.point;
+        struct point vfinger = inverse_point(mouse, im->screen->frame_size);
+        enum android_motionevent_action action = down
+                                               ? AMOTION_EVENT_ACTION_DOWN
+                                               : AMOTION_EVENT_ACTION_UP;
+        if (!simulate_virtual_finger(im, action, vfinger)) {
+            return;
         }
+        im->vfinger_down = down;
     }
 }
 
@@ -568,7 +711,8 @@ convert_mouse_wheel(const SDL_MouseWheelEvent *from, struct screen *screen,
 
     struct position position = {
         .screen_size = screen->frame_size,
-        .point = screen_convert_to_frame_coords(screen, mouse_x, mouse_y),
+        .point = screen_convert_window_to_frame_coords(screen,
+                                                       mouse_x, mouse_y),
     };
 
     to->type = CONTROL_MSG_TYPE_INJECT_SCROLL_EVENT;
