@@ -8,6 +8,8 @@
 #include <SDL2/SDL.h>
 
 #ifdef _WIN32
+// not needed here, but winsock2.h must never be included AFTER windows.h
+# include <winsock2.h>
 # include <windows.h>
 #endif
 
@@ -32,7 +34,7 @@
 #include "util/log.h"
 #include "util/net.h"
 
-static struct server server = SERVER_INITIALIZER;
+static struct server server;
 static struct screen screen = SCREEN_INITIALIZER;
 static struct fps_counter fps_counter;
 static struct video_buffer video_buffer;
@@ -46,7 +48,14 @@ static struct input_manager input_manager = {
     .controller = &controller,
     .video_buffer = &video_buffer,
     .screen = &screen,
-    .prefer_text = false, // initialized later
+    .repeat = 0,
+
+    // initialized later
+    .prefer_text = false,
+    .sdl_shortcut_mods = {
+        .data = {0},
+        .count = 0,
+    },
 };
 
 #ifdef _WIN32
@@ -63,7 +72,8 @@ BOOL WINAPI windows_ctrl_handler(DWORD ctrl_type) {
 
 // init SDL and set appropriate hints
 static bool
-sdl_init_and_configure(bool display, const char *render_driver) {
+sdl_init_and_configure(bool display, const char *render_driver,
+                       bool disable_screensaver) {
     uint32_t flags = display ? SDL_INIT_VIDEO : SDL_INIT_EVENTS;
     if (SDL_Init(flags)) {
         LOGC("Could not initialize SDL: %s", SDL_GetError());
@@ -112,8 +122,13 @@ sdl_init_and_configure(bool display, const char *render_driver) {
         LOGW("Could not disable minimize on focus loss");
     }
 
-    // Do not disable the screensaver when scrcpy is running
-    SDL_EnableScreenSaver();
+    if (disable_screensaver) {
+        LOGD("Screensaver disabled");
+        SDL_DisableScreenSaver();
+    } else {
+        LOGD("Screensaver enabled");
+        SDL_EnableScreenSaver();
+    }
 
     return true;
 }
@@ -155,7 +170,7 @@ enum event_result {
 };
 
 static enum event_result
-handle_event(SDL_Event *event, bool control) {
+handle_event(SDL_Event *event, const struct scrcpy_options *options) {
     switch (event->type) {
         case EVENT_STREAM_STOPPED:
             LOGD("Video stream stopped");
@@ -177,7 +192,7 @@ handle_event(SDL_Event *event, bool control) {
             screen_handle_window_event(&screen, &event->window);
             break;
         case SDL_TEXTINPUT:
-            if (!control) {
+            if (!options->control) {
                 break;
             }
             input_manager_process_text_input(&input_manager, &event->text);
@@ -186,16 +201,16 @@ handle_event(SDL_Event *event, bool control) {
         case SDL_KEYUP:
             // some key events do not interact with the device, so process the
             // event even if control is disabled
-            input_manager_process_key(&input_manager, &event->key, control);
+            input_manager_process_key(&input_manager, &event->key);
             break;
         case SDL_MOUSEMOTION:
-            if (!control) {
+            if (!options->control) {
                 break;
             }
             input_manager_process_mouse_motion(&input_manager, &event->motion);
             break;
         case SDL_MOUSEWHEEL:
-            if (!control) {
+            if (!options->control) {
                 break;
             }
             input_manager_process_mouse_wheel(&input_manager, &event->wheel);
@@ -204,8 +219,7 @@ handle_event(SDL_Event *event, bool control) {
         case SDL_MOUSEBUTTONUP:
             // some mouse events do not interact with the device, so process
             // the event even if control is disabled
-            input_manager_process_mouse_button(&input_manager, &event->button,
-                                               control);
+            input_manager_process_mouse_button(&input_manager, &event->button);
             break;
         case SDL_FINGERMOTION:
         case SDL_FINGERDOWN:
@@ -213,7 +227,7 @@ handle_event(SDL_Event *event, bool control) {
             input_manager_process_touch(&input_manager, &event->tfinger);
             break;
         case SDL_DROPFILE: {
-            if (!control) {
+            if (!options->control) {
                 break;
             }
             file_handler_action_t action;
@@ -230,16 +244,15 @@ handle_event(SDL_Event *event, bool control) {
 }
 
 static bool
-event_loop(bool display, bool control) {
-    (void) display;
+event_loop(const struct scrcpy_options *options) {
 #ifdef CONTINUOUS_RESIZING_WORKAROUND
-    if (display) {
+    if (options->display) {
         SDL_AddEventWatch(event_watcher, NULL);
     }
 #endif
     SDL_Event event;
     while (SDL_WaitEvent(&event)) {
-        enum event_result result = handle_event(&event, control);
+        enum event_result result = handle_event(&event, options);
         switch (result) {
             case EVENT_RESULT_STOPPED_BY_USER:
                 return true;
@@ -291,6 +304,19 @@ av_log_callback(void *avcl, int level, const char *fmt, va_list vl) {
 
 bool
 scrcpy(const struct scrcpy_options *options) {
+    if (!server_init(&server)) {
+        return false;
+    }
+
+    bool server_started = false;
+    bool fps_counter_initialized = false;
+    bool video_buffer_initialized = false;
+    bool file_handler_initialized = false;
+    bool recorder_initialized = false;
+    bool stream_started = false;
+    bool controller_initialized = false;
+    bool controller_started = false;
+
     bool record = !!options->record_filename;
     struct server_params params = {
         .log_level = options->log_level,
@@ -305,23 +331,17 @@ scrcpy(const struct scrcpy_options *options) {
         .show_touches = options->show_touches,
         .stay_awake = options->stay_awake,
         .codec_options = options->codec_options,
+        .encoder_name = options->encoder_name,
         .force_adb_forward = options->force_adb_forward,
     };
     if (!server_start(&server, options->serial, &params)) {
-        return false;
+        goto end;
     }
 
-    bool ret = false;
+    server_started = true;
 
-    bool fps_counter_initialized = false;
-    bool video_buffer_initialized = false;
-    bool file_handler_initialized = false;
-    bool recorder_initialized = false;
-    bool stream_started = false;
-    bool controller_initialized = false;
-    bool controller_started = false;
-
-    if (!sdl_init_and_configure(options->display, options->render_driver)) {
+    if (!sdl_init_and_configure(options->display, options->render_driver,
+                                options->disable_screensaver)) {
         goto end;
     }
 
@@ -408,7 +428,7 @@ scrcpy(const struct scrcpy_options *options) {
                                    options->window_y, options->window_width,
                                    options->window_height,
                                    options->window_borderless,
-                                   options->rotation, options-> mipmaps)) {
+                                   options->rotation, options->mipmaps)) {
             goto end;
         }
 
@@ -427,9 +447,9 @@ scrcpy(const struct scrcpy_options *options) {
         }
     }
 
-    input_manager.prefer_text = options->prefer_text;
+    input_manager_init(&input_manager, options);
 
-    ret = event_loop(options->display, options->control);
+    bool ret = event_loop(options);
     LOGD("quit...");
 
     screen_destroy(&screen);
@@ -450,8 +470,10 @@ end:
         fps_counter_interrupt(&fps_counter);
     }
 
-    // shutdown the sockets and kill the server
-    server_stop(&server);
+    if (server_started) {
+        // shutdown the sockets and kill the server
+        server_stop(&server);
+    }
 
     // now that the sockets are shutdown, the stream and controller are
     // interrupted, we can join them
