@@ -9,8 +9,6 @@
 #include "common.h"
 #include "opengl.h"
 
-#define WINDOW_POSITION_UNDEFINED (-0x8000)
-
 struct video_buffer;
 
 struct screen {
@@ -76,7 +74,7 @@ void
 screen_init(struct screen *screen);
 
 // initialize screen, create window, renderer and texture (window is hidden)
-// window_x and window_y accept WINDOW_POSITION_UNDEFINED
+// window_x and window_y accept SC_WINDOW_POSITION_UNDEFINED
 bool
 screen_init_rendering(struct screen *screen, const char *window_title,
                       struct size frame_size, bool always_on_top,
@@ -126,7 +124,14 @@ screen_handle_window_event(struct screen *screen, const SDL_WindowEvent *event);
 // convert point from window coordinates to frame coordinates
 // x and y are expressed in pixels
 struct point
-screen_convert_to_frame_coords(struct screen *screen, int32_t x, int32_t y);
+screen_convert_window_to_frame_coords(struct screen *screen,
+                                      int32_t x, int32_t y);
+
+// convert point from drawable coordinates to frame coordinates
+// x and y are expressed in pixels
+struct point
+screen_convert_drawable_to_frame_coords(struct screen *screen,
+                                        int32_t x, int32_t y);
 
 // Convert coordinates from window to drawable.
 // Events are expressed in window coordinates, but content is expressed in
