@@ -11,6 +11,7 @@
 
 #include "config.h"
 #include "command.h"
+#include "util/lock.h"
 #include "util/log.h"
 #include "util/net.h"
 #include "util/str_util.h"
@@ -143,7 +144,7 @@ listen_on_port(uint16_t port) {
 
 static bool
 enable_tunnel_reverse_any_port(struct server *server,
-                               struct port_range port_range) {
+                               struct sc_port_range port_range) {
     uint16_t port = port_range.first;
     for (;;) {
         if (!enable_tunnel_reverse(server->serial, port)) {
@@ -189,7 +190,7 @@ enable_tunnel_reverse_any_port(struct server *server,
 
 static bool
 enable_tunnel_forward_any_port(struct server *server,
-                               struct port_range port_range) {
+                               struct sc_port_range port_range) {
     server->tunnel_forward = true;
     uint16_t port = port_range.first;
     for (;;) {
@@ -201,7 +202,7 @@ enable_tunnel_forward_any_port(struct server *server,
 
         if (port < port_range.last) {
             LOGW("Could not forward port %" PRIu16", retrying on %" PRIu16,
-                 port, port + 1);
+                 port, (uint16_t) (port + 1));
             port++;
             continue;
         }
@@ -217,7 +218,7 @@ enable_tunnel_forward_any_port(struct server *server,
 }
 
 static bool
-enable_tunnel_any_port(struct server *server, struct port_range port_range,
+enable_tunnel_any_port(struct server *server, struct sc_port_range port_range,
                        bool force_adb_forward) {
     if (!force_adb_forward) {
         // Attempt to use "adb reverse"
@@ -294,6 +295,7 @@ execute_server(struct server *server, const struct server_params *params) {
         params->show_touches ? "true" : "false",
         params->stay_awake ? "true" : "false",
         params->codec_options ? params->codec_options : "-",
+        params->encoder_name ? params->encoder_name : "-",
     };
 #ifdef SERVER_DEBUGGER
     LOGI("Server debugger waiting for a client on device port "
@@ -352,15 +354,51 @@ close_socket(socket_t socket) {
     }
 }
 
-void
+bool
 server_init(struct server *server) {
-    *server = (struct server) SERVER_INITIALIZER;
+    server->serial = NULL;
+    server->process = PROCESS_NONE;
+    server->wait_server_thread = NULL;
+    atomic_flag_clear_explicit(&server->server_socket_closed,
+                               memory_order_relaxed);
+
+    server->mutex = SDL_CreateMutex();
+    if (!server->mutex) {
+        return false;
+    }
+
+    server->process_terminated_cond = SDL_CreateCond();
+    if (!server->process_terminated_cond) {
+        SDL_DestroyMutex(server->mutex);
+        return false;
+    }
+
+    server->process_terminated = false;
+
+    server->server_socket = INVALID_SOCKET;
+    server->video_socket = INVALID_SOCKET;
+    server->control_socket = INVALID_SOCKET;
+
+    server->port_range.first = 0;
+    server->port_range.last = 0;
+    server->local_port = 0;
+
+    server->tunnel_enabled = false;
+    server->tunnel_forward = false;
+
+    return true;
 }
 
 static int
 run_wait_server(void *data) {
     struct server *server = data;
     cmd_simple_wait(server->process, NULL); // ignore exit code
+
+    mutex_lock(server->mutex);
+    server->process_terminated = true;
+    cond_signal(server->process_terminated_cond);
+    mutex_unlock(server->mutex);
+
     // no need for synchronization, server_socket is initialized before this
     // thread was created
     if (server->server_socket != INVALID_SOCKET
@@ -492,17 +530,39 @@ server_stop(struct server *server) {
 
     assert(server->process != PROCESS_NONE);
 
-    cmd_terminate(server->process);
-
     if (server->tunnel_enabled) {
         // ignore failure
         disable_tunnel(server);
     }
 
+    // Give some delay for the server to terminate properly
+    mutex_lock(server->mutex);
+    int r = 0;
+    if (!server->process_terminated) {
+#define WATCHDOG_DELAY_MS 1000
+        r = cond_wait_timeout(server->process_terminated_cond,
+                              server->mutex,
+                              WATCHDOG_DELAY_MS);
+    }
+    mutex_unlock(server->mutex);
+
+    // After this delay, kill the server if it's not dead already.
+    // On some devices, closing the sockets is not sufficient to wake up the
+    // blocking calls while the device is asleep.
+    if (r == SDL_MUTEX_TIMEDOUT) {
+        // FIXME There is a race condition here: there is a small chance that
+        // the process is already terminated, and the PID assigned to a new
+        // process.
+        LOGW("Killing the server...");
+        cmd_terminate(server->process);
+    }
+
     SDL_WaitThread(server->wait_server_thread, NULL);
 }
 
 void
 server_destroy(struct server *server) {
     SDL_free(server->serial);
+    SDL_DestroyCond(server->process_terminated_cond);
+    SDL_DestroyMutex(server->mutex);
 }
