@@ -9,18 +9,20 @@ static void test_serialize_inject_keycode(void) {
         .inject_keycode = {
             .action = AKEY_EVENT_ACTION_UP,
             .keycode = AKEYCODE_ENTER,
+            .repeat = 5,
             .metastate = AMETA_SHIFT_ON | AMETA_SHIFT_LEFT_ON,
         },
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
-    assert(size == 10);
+    assert(size == 14);
 
     const unsigned char expected[] = {
         CONTROL_MSG_TYPE_INJECT_KEYCODE,
         0x01, // AKEY_EVENT_ACTION_UP
         0x00, 0x00, 0x00, 0x42, // AKEYCODE_ENTER
+        0x00, 0x00, 0x00, 0X05, // repeat
         0x00, 0x00, 0x00, 0x41, // AMETA_SHIFT_ON | AMETA_SHIFT_LEFT_ON
     };
     assert(!memcmp(buf, expected, sizeof(expected)));
@@ -34,13 +36,13 @@ static void test_serialize_inject_text(void) {
         },
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
-    assert(size == 16);
+    assert(size == 18);
 
     const unsigned char expected[] = {
         CONTROL_MSG_TYPE_INJECT_TEXT,
-        0x00, 0x0d, // text length
+        0x00, 0x00, 0x00, 0x0d, // text length
         'h', 'e', 'l', 'l', 'o', ',', ' ', 'w', 'o', 'r', 'l', 'd', '!', // text
     };
     assert(!memcmp(buf, expected, sizeof(expected)));
@@ -54,15 +56,17 @@ static void test_serialize_inject_text_long(void) {
     text[CONTROL_MSG_INJECT_TEXT_MAX_LENGTH] = '\0';
     msg.inject_text.text = text;
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
-    assert(size == 3 + CONTROL_MSG_INJECT_TEXT_MAX_LENGTH);
+    assert(size == 5 + CONTROL_MSG_INJECT_TEXT_MAX_LENGTH);
 
-    unsigned char expected[3 + CONTROL_MSG_INJECT_TEXT_MAX_LENGTH];
+    unsigned char expected[5 + CONTROL_MSG_INJECT_TEXT_MAX_LENGTH];
     expected[0] = CONTROL_MSG_TYPE_INJECT_TEXT;
-    expected[1] = 0x01;
-    expected[2] = 0x2c; // text length (16 bits)
-    memset(&expected[3], 'a', CONTROL_MSG_INJECT_TEXT_MAX_LENGTH);
+    expected[1] = 0x00;
+    expected[2] = 0x00;
+    expected[3] = 0x01;
+    expected[4] = 0x2c; // text length (32 bits)
+    memset(&expected[5], 'a', CONTROL_MSG_INJECT_TEXT_MAX_LENGTH);
 
     assert(!memcmp(buf, expected, sizeof(expected)));
 }
@@ -88,7 +92,7 @@ static void test_serialize_inject_touch_event(void) {
         },
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 28);
 
@@ -123,7 +127,7 @@ static void test_serialize_inject_scroll_event(void) {
         },
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 21);
 
@@ -142,7 +146,7 @@ static void test_serialize_back_or_screen_on(void) {
         .type = CONTROL_MSG_TYPE_BACK_OR_SCREEN_ON,
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 1);
 
@@ -157,7 +161,7 @@ static void test_serialize_expand_notification_panel(void) {
         .type = CONTROL_MSG_TYPE_EXPAND_NOTIFICATION_PANEL,
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 1);
 
@@ -172,7 +176,7 @@ static void test_serialize_collapse_notification_panel(void) {
         .type = CONTROL_MSG_TYPE_COLLAPSE_NOTIFICATION_PANEL,
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 1);
 
@@ -187,7 +191,7 @@ static void test_serialize_get_clipboard(void) {
         .type = CONTROL_MSG_TYPE_GET_CLIPBOARD,
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 1);
 
@@ -206,14 +210,14 @@ static void test_serialize_set_clipboard(void) {
         },
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
-    assert(size == 17);
+    assert(size == 19);
 
     const unsigned char expected[] = {
         CONTROL_MSG_TYPE_SET_CLIPBOARD,
         1, // paste
-        0x00, 0x0d, // text length
+        0x00, 0x00, 0x00, 0x0d, // text length
         'h', 'e', 'l', 'l', 'o', ',', ' ', 'w', 'o', 'r', 'l', 'd', '!', // text
     };
     assert(!memcmp(buf, expected, sizeof(expected)));
@@ -227,7 +231,7 @@ static void test_serialize_set_screen_power_mode(void) {
         },
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 2);
 
@@ -243,7 +247,7 @@ static void test_serialize_rotate_device(void) {
         .type = CONTROL_MSG_TYPE_ROTATE_DEVICE,
     };
 
-    unsigned char buf[CONTROL_MSG_SERIALIZED_MAX_SIZE];
+    unsigned char buf[CONTROL_MSG_MAX_SIZE];
     int size = control_msg_serialize(&msg, buf);
     assert(size == 1);
 
@@ -253,7 +257,10 @@ static void test_serialize_rotate_device(void) {
     assert(!memcmp(buf, expected, sizeof(expected)));
 }
 
-int main(void) {
+int main(int argc, char *argv[]) {
+    (void) argc;
+    (void) argv;
+
     test_serialize_inject_keycode();
     test_serialize_inject_text();
     test_serialize_inject_text_long();
