@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 #
 # This script generates the scrcpy binary "manually" (without gradle).
 #
@@ -12,10 +12,10 @@
 set -e
 
 SCRCPY_DEBUG=false
-SCRCPY_VERSION_NAME=1.14
+SCRCPY_VERSION_NAME=1.17
 
-PLATFORM=${ANDROID_PLATFORM:-29}
-BUILD_TOOLS=${ANDROID_BUILD_TOOLS:-29.0.2}
+PLATFORM=${ANDROID_PLATFORM:-30}
+BUILD_TOOLS=${ANDROID_BUILD_TOOLS:-30.0.0}
 
 BUILD_DIR="$(realpath ${BUILD_DIR:-build_manual})"
 CLASSES_DIR="$BUILD_DIR/classes"
@@ -42,6 +42,8 @@ echo "Generating java from aidl..."
 cd "$SERVER_DIR/src/main/aidl"
 "$ANDROID_HOME/build-tools/$BUILD_TOOLS/aidl" -o"$CLASSES_DIR" \
     android/view/IRotationWatcher.aidl
+"$ANDROID_HOME/build-tools/$BUILD_TOOLS/aidl" -o"$CLASSES_DIR" \
+    android/content/IOnPrimaryClipChangedListener.aidl
 
 echo "Compiling java sources..."
 cd ../java
@@ -55,6 +57,7 @@ cd "$CLASSES_DIR"
 "$ANDROID_HOME/build-tools/$BUILD_TOOLS/dx" --dex \
     --output "$BUILD_DIR/classes.dex" \
     android/view/*.class \
+    android/content/*.class \
     com/genymobile/scrcpy/*.class \
     com/genymobile/scrcpy/wrappers/*.class
 
